// The sample contained herein is provided to you "AS IS".
//
// It is furnished by Add-in Express as a simple example and has not been 
// thoroughly tested under all conditions. Add-in Express, therefore, cannot 
// guarantee its reliability, serviceability or functionality. 

unit uUtils;

interface

uses
  Windows, Dialogs, Outlook2000, adxolFormsManager, ActiveX, Forms;

type
  ProjectConsts = class
  public
    class function DevCode: string; // = "D7";
    class function HowtoName: string; // = "Changing Form Layout";
    class function ProjectShortName: string; // = "Layouts";
    class function MessageCaption: string; // = HowtoName;
    class function HowtoProjectName: string; // = ProjectShortName + DevCode;
    class function CommandBarName: string; // = HowtoProjectName;
  end;

  Strings = class
    class function AddinModuleOnError: string; // = "An error has been handled in the AddinModule.OnError event";
    class function FormsManagerOnError: string; // = "An error has been handled in the ADXOlFormsManagger.OnError event";
    class function SubjectOfCurrentEmail: string; // = "The subject of the current e-mail is: \n {0}";
  end;

  Common = class
  public
    class function IndexOfString(source: string; str: string): integer;
    class procedure ShowError(mes: string);
    class procedure ShowMessage(mes: string);
  end;

  OutlookPane = (
    olFolderList = 2,
    olNavigationPane = 4,
    olOutlookBar = 1,
    olPreview = 3,
    olToDoPane = 5);

  HostVersion = (vUnknown, v2000, v2002, v2003, v2007);

  OutlookUtils = class
  public
    class procedure InitOutlookVersion(OutlookApp: OutlookApplication);
    class function OutlookVersion: HostVersion;
    class function RefreshCurrentFolderExecuting: boolean;
    class function ExistsExplorerAdvancedFormRegion(AExplorer: Explorer;
      ExplorerLayout: TadxOlExplorerLayout; var failureMessage: string): boolean;
    class procedure RefreshCurrentFolder(AExplorer: Explorer);
    class procedure ClearFolderWebViewProperties(AFolder: MAPIFolder);
    class procedure SetFolderWebViewProperties(Folder: MAPIFolder; WebViewURL: string; WebViewOn: boolean);
    class function IsExplorerPaneVisible(AExplorer: Explorer; OlPane: TOleEnum): boolean;
    class procedure SetExplorerFolder(AExplorer: Explorer; Folder: MAPIFolder);

    class function GetExplorerSelectionItem(AExplorer: Explorer): IDispatch;

  end;

  ADXUtils = class
  public
    class procedure SetExplorerLayout(Item: TadxOlFormsCollectionItem;
      explorerLayoutText: string; OutlookApp: _Application);
    class procedure SetInspectorLayout(Item: TadxOlFormsCollectionItem;
      inspectorLayoutText: string; OutlookApp: _Application);
    class function ConvertToADXExplorerLayout(explorerLayoutText: string): TadxOlExplorerLayout;
    class function ConvertToADXOlInspectorLayout(inspectorLayoutText: string): TadxolInspectorLayout;
    class function ExistsVisibleForm(Item: TadxOlFormsCollectionItem): boolean;
    class function ADXHTMLFileName: string;
  end;


implementation

var
  OutlookUtilsOutlookVersion: HostVersion;
  OutlookUtilsRefreshCurrentFolderExecuting: boolean;



{ ProjectConsts }

class function ProjectConsts.CommandBarName: string;
begin
  result := HowtoProjectName;
end;

class function ProjectConsts.DevCode: string;
begin
  result := 'D7';
end;

class function ProjectConsts.HowtoName: string;
begin
  result := 'Changing Form Layout';
end;

class function ProjectConsts.HowtoProjectName: string;
begin
  result := ProjectShortName + DevCode;
end;

class function ProjectConsts.MessageCaption: string;
begin
  result := HowtoName;
end;

class function ProjectConsts.ProjectShortName: string;
begin
  result := 'Layouts';
end;

{ Strings }

class function Strings.AddinModuleOnError: string;
begin
  result := 'An error has been handled in the AddinModule.OnError event';
end;

class function Strings.FormsManagerOnError: string;
begin
  result := 'An error has been handled in the ADXOlFormsManagger.OnError event';
end;

class function Strings.SubjectOfCurrentEmail: string;
begin
  result := 'The subject of the current e-mail is: '#13#10'%s';
end;

{ Common }

class function Common.IndexOfString(source, str: string): integer;
begin
  result := Pos(str, source);
end;

class procedure Common.ShowError(mes: string);
begin
  MessageBox(0, PChar(mes), PChar(ProjectConsts.MessageCaption), MB_OK or MB_ICONERROR or MB_APPLMODAL);
end;

class procedure Common.ShowMessage(mes: string);
begin
  MessageBox(0, PChar(mes), PChar(ProjectConsts.MessageCaption), MB_OK or MB_APPLMODAL);
end;

{ OutlookUtils }

class procedure OutlookUtils.ClearFolderWebViewProperties(
  AFolder: MAPIFolder);
begin
  SetFolderWebViewProperties(AFolder, '', false);
end;

class function OutlookUtils.ExistsExplorerAdvancedFormRegion(
  AExplorer: Explorer; ExplorerLayout: TadxOlExplorerLayout;
  var failureMessage: string): boolean;
begin
  result := false;
  case (ExplorerLayout) of
    elLeftSubpane,
    elRightSubpane,
    elTopSubpane,
    elBottomSubpane: begin
      result := true;
      exit;
    end;

    elWebViewPane: begin
      result := true;
      exit;
    end;

    elBottomNavigationPane: begin
      case (OutlookUtils.OutlookVersion) of
        v2000,
        v2002:
          result := OutlookUtils.IsExplorerPaneVisible(AExplorer, ord(olFolderList));
        v2003,
        v2007:
          result := OutlookUtils.IsExplorerPaneVisible(AExplorer, 4{olNavigationPane});
      end;

      if (not result) then begin
        case (OutlookUtils.OutlookVersion) of
          v2000,
          v2002:
            failureMessage := 'Please, enable the Folder List. See the View | Folder List menu.';
          v2003,
          v2007:
            failureMessage := 'Please, enable the Navigation Pane. See the View | Navigation Pane menu or Press Alt + F1.';
        end;
      end;
      exit;
    end;

    elBottomOutlookBar: begin
      case (OutlookUtils.OutlookVersion) of
        v2000,
        v2002: begin
            result := IsExplorerPaneVisible(AExplorer, ord(olOutlookBar));
            if (not result) then begin
              failureMessage := 'Please, enable the Outlook Bar. See the View | Outlook Bar menu.';
            end;
            exit;
          end;
        v2003,
        v2007: begin
            failureMessage := 'Outlook Bar is available in 2000 and XP versions only.';
            exit;
          end;
      else
        exit;
      end;
    end;

    elBottomTodoBar: begin
      case (OutlookUtils.OutlookVersion) of
        v2000,
        v2002,
        v2003:
          failureMessage := 'To-Do bar is available starting from version 2007.';
        v2007: begin
          result := IsExplorerPaneVisible(AExplorer, 5 {olToDoPane});
          if (not result) then begin
            failureMessage := 'Please, enable To-Do Pane. See the View | To-Do Bar menu or press Alt + F2.';
          end;
        end;
      else
        exit;
      end;
    end;

    elBottomReadingPane,
    elLeftReadingPane,
    elRightReadingPane,
    elTopReadingPane: begin
      result := IsExplorerPaneVisible(AExplorer, ord(olPreview));
      if (not result) then begin
        case (OutlookUtils.OutlookVersion) of
          v2000,
          v2002:
            failureMessage := 'Please, enable the Preview Pane. See the View | Preview Pane menu.';
          v2003,
          v2007:
            failureMessage := 'Please, enable the Reading Pane. See the View | Reading Pane menu.';
        end;
      end;
    end;
    else
      exit;
  end;
end;

class function OutlookUtils.GetExplorerSelectionItem(
  AExplorer: Explorer): IDispatch;
var
  sel: Selection;
begin
  result := nil;
  sel := AExplorer.Selection;
  try
    if (sel.Count > 0) then begin
      result := sel.Item(1);
    end;
  finally
    sel := nil;
  end;
end;

class procedure OutlookUtils.InitOutlookVersion(
  OutlookApp: _Application);
var
  hostVersion: string;
begin
  if (OutlookApp <> nil) then begin
      hostVersion := OutlookApp.Version;

      if (Common.IndexOfString(hostVersion, '9.0') >= 0) then begin
        OutlookUtilsOutlookVersion := v2000;
      end;

      if (Common.IndexOfString(hostVersion, '10.0') >= 0) then begin
        OutlookUtilsOutlookVersion := v2002;
      end;

      if (Common.IndexOfString(hostVersion, '11.0') >= 0) then begin
        OutlookUtilsOutlookVersion := v2003;
      end;

      if (Common.IndexOfString(hostVersion, '12.0') >= 0) then begin
        OutlookUtilsOutlookVersion := v2007;
      end;
  end;
end;

class function OutlookUtils.IsExplorerPaneVisible(AExplorer: Explorer;
  OlPane: TOleEnum): boolean;
begin
  result := AExplorer.IsPaneVisible(OlPane);
end;

class function OutlookUtils.OutlookVersion: HostVersion;
begin
  result := OutlookUtilsOutlookVersion; 
end;

class procedure OutlookUtils.RefreshCurrentFolder(AExplorer: Explorer);
var
  OutlookApplication: _Application;
  currentFolder: MAPIFolder;
  nmSpace:  NameSpace;
  outboxFolder: MAPIFolder;
begin
  OutlookUtilsRefreshCurrentFolderExecuting := true;
  try
    OutlookApplication := AExplorer.Application;
    currentFolder := AExplorer.CurrentFolder;
    nmSpace := OutlookApplication.GetNamespace('MAPI');
    outboxFolder := nmSpace.GetDefaultFolder(olFolderOutbox);
    try
      OutlookUtils.SetExplorerFolder(AExplorer, outboxFolder);
      Application.ProcessMessages;
      OutlookUtils.SetExplorerFolder(AExplorer, currentFolder);
    finally
      nmSpace := nil;
      currentFolder := nil;
      outboxFolder := nil;
      OutlookApplication := nil;
    end;
  finally
      OutlookUtilsRefreshCurrentFolderExecuting := false;
  end;
end;

class function OutlookUtils.RefreshCurrentFolderExecuting: boolean;
begin
  result := OutlookUtilsRefreshCurrentFolderExecuting;
end;

class procedure OutlookUtils.SetExplorerFolder(AExplorer: Explorer;
  Folder: MAPIFolder);
begin
  AExplorer.CurrentFolder := Folder;
end;

class procedure OutlookUtils.SetFolderWebViewProperties(Folder: MAPIFolder;
  WebViewURL: string; WebViewOn: boolean);
begin
  if (Folder <> nil) then begin
    Folder.WebViewURL := WebViewURL;
    Folder.WebViewOn := WebViewOn;
  end;
end;

{ ADXUtils }

class function ADXUtils.ADXHTMLFileName: string;
begin
  result := 'ADXOlFormGeneral.html';
end;

class function ADXUtils.ConvertToADXExplorerLayout(
  explorerLayoutText: string): TadxolExplorerLayout;
begin
  result := elUnknown;
  if explorerLayoutText = 'elUnknown'              then result := elUnknown;
  if explorerLayoutText = 'elTopSubpane'           then result := elTopSubpane;
  if explorerLayoutText = 'elBottomSubpane'        then result := elBottomSubpane;
  if explorerLayoutText = 'elRightSubpane'         then result := elRightSubpane;
  if explorerLayoutText = 'elWebViewPane'          then result := elWebViewPane;
  if explorerLayoutText = 'elLeftSubpane'          then result := elLeftSubpane;
  if explorerLayoutText = 'elBottomOutlookBar'     then result := elBottomOutlookBar;
  if explorerLayoutText = 'elBottomNavigationPane' then result := elBottomNavigationPane;
  if explorerLayoutText = 'elBottomTodoBar'        then result := elBottomTodoBar;
  if explorerLayoutText = 'elTopReadingPane'       then result := elTopReadingPane;
  if explorerLayoutText = 'elBottomReadingPane'    then result := elBottomReadingPane;
  if explorerLayoutText = 'elLeftReadingPane'      then result := elLeftReadingPane;
  if explorerLayoutText = 'elRightReadingPane'     then result := elRightReadingPane;
end;

class function ADXUtils.ConvertToADXOlInspectorLayout(
  inspectorLayoutText: string): TadxolInspectorLayout;
begin
  result := ilUnknown;
  if inspectorLayoutText = 'ilUnknown'              then result := ilUnknown;
  if inspectorLayoutText = 'ilTopSubpane'           then result := ilTopSubpane;
  if inspectorLayoutText = 'ilBottomSubpane'        then result := ilBottomSubpane;
  if inspectorLayoutText = 'ilRightSubpane'         then result := ilRightSubpane;
  if inspectorLayoutText = 'ilLeftSubpane'          then result := ilLeftSubpane;
end;

class function ADXUtils.ExistsVisibleForm(Item: TadxOlFormsCollectionItem): boolean;
var
  i: integer;
begin
  result := false;
  for i := 0 to Item.FormInstanceCount - 1 do begin
    if (Item.FormInstances[i].Visible) then begin
      result := true;
      break;
    end;
  end;
end;

class procedure ADXUtils.SetExplorerLayout(Item: TadxOlFormsCollectionItem;
  explorerLayoutText: string; OutlookApp: _Application);
var
  expl: Explorer;
  newExplorerLayout: TadxOlExplorerLayout;
  failureMessage: string;
  currentFolder: MAPIFolder;
  NeedClearFolderWebViewProperties: boolean;
begin
  expl := OutlookApp.ActiveExplorer;
  try
    newExplorerLayout := ADXUtils.ConvertToADXExplorerLayout(explorerLayoutText);
    if (newExplorerLayout <> elUnknown) then begin
      failureMessage := '';
      if (not OutlookUtils.ExistsExplorerAdvancedFormRegion(expl, newExplorerLayout, failureMessage)) then begin
        Common.ShowMessage(failureMessage);
      end;
       Item.ExplorerLayout := newExplorerLayout;
       if (not ADXUtils.ExistsVisibleForm(Item)) then begin
        if (newExplorerLayout = elWebViewPane) then begin
          OutlookUtils.RefreshCurrentFolder(expl);
          Item.ApplyTo(expl);
        end
        else begin
            currentFolder := expl.CurrentFolder;
            try
                NeedClearFolderWebViewProperties :=
                    (currentFolder.WebViewOn and (Common.IndexOfString(currentFolder.WebViewURL, ADXHTMLFileName) >= 0));

                if (NeedClearFolderWebViewProperties) then begin
                  OutlookUtils.ClearFolderWebViewProperties(currentFolder);
                  OutlookUtils.RefreshCurrentFolder(expl);
                  Item.ApplyTo(expl);
                end
                else begin
                  Item.ApplyTo(expl);
                end;
            finally
              currentFolder := nil;
            end;
        end;
      end;
    end;
  finally
    expl := nil;
  end
end;

class procedure ADXUtils.SetInspectorLayout(
  Item: TadxOlFormsCollectionItem; inspectorLayoutText: string;
  OutlookApp: _Application);
var
  newInspectorLayout: TadxOlInspectorLayout;
  activeInspector: Inspector;
begin
  newInspectorLayout := ADXUtils.ConvertToADXOlInspectorLayout(inspectorLayoutText);
  if (newInspectorLayout <> ilUnknown) then begin
    Item.InspectorLayout := newInspectorLayout;
  end;
  if (not ADXUtils.ExistsVisibleForm(Item)) then begin
      activeInspector := OutlookApp.ActiveInspector;
      try
       if (activeInspector <> nil) then begin
           Item.ApplyTo(activeInspector);
       end;
      finally
        activeInspector := nil;
      end;
  end;
end;

end.
